import fnmatch
import os

from .sass import compile_str

from watchdog.events import FileSystemEventHandler

class FileSystemEvents(FileSystemEventHandler):
    """Handler for all filesystem events."""

    def __init__(self, source_path, dest_path):
        super(FileSystemEvents, self).__init__()

        self._source_path = source_path
        self._dest_path = dest_path

    def get_scss_files(self, skip_partials=True, with_source_path=False):
        """Gets all SCSS files in the source directory.

        :param bool skip_partials: If True, partials will be ignored. Otherwise,
                                   all SCSS files, including ones that begin
                                   with '_' will be returned.
        :param boom with_source_path: If true, the `source_path` will be added
                                      to all of the paths. Otherwise, it will
                                      be stripped.
        :returns: A list of the SCSS files in the source directory

        """
        scss_files = []

        for root, dirs, files in os.walk(self._source_path):
            for filename in fnmatch.filter(files, "*.scss"):
                if filename.startswith("_") and skip_partials:
                    continue

                full_path = os.path.join(root, filename)
                if not with_source_path:
                    full_path = full_path.split(self._source_path)[1]

                    if full_path.startswith("/"):
                        full_path = full_path[1:]

                scss_files.append(full_path)

        return scss_files

    def compile_scss(self):
        if not os.path.exists(self._dest_path):
            os.makedirs(self._dest_path)

        for scss_file in self.get_scss_files():
            # Read in the source SCSS file contents
            contents = ""
            with open(os.path.join(self._source_path, scss_file)) as open_file:
                contents = open_file.read()

            # Write out the CSS file
            css_filename = os.path.splitext(scss_file)[0] + ".css"

            with open(os.path.join(self._dest_path, css_filename), 'w') as css_file:
                css_file.write(compile_str(contents))

    def on_any_event(self, event):
        self.compile_scss()



from . import version
from .api import watch

__title__ = 'pylibsass'
__version__ = version.__version__
__author__ = 'Ryan Senkbeil'
__license__ = 'MIT'
__copyright__ = 'Copyright 2013 Ryan Senkbeil'



from ctypes import *

class SassOptions(Structure):
    _fields_ = [
        ("output_style", c_int),
        ("source_comments", c_int),
        ("include_paths", c_char_p),
        ("image_path", c_char_p),
    ]

class SassContext(Structure):
    _fields_ = [
        ("source_string", c_char_p),
        ("output_string", c_char_p),
        ("sass_options", SassOptions),
        ("error_status", c_int),
        ("error_message", c_char_p),
    ]

class SassFileContext(Structure):
    _fields_ = [
        ("input_path", c_char_p),
        ("output_string", c_char_p),
        ("sass_options", SassOptions),
        ("error_status", c_int),
        ("error_message", c_char_p),
    ]

class SassFolderContext(Structure):
    _fields_ = [
        ("search_path", c_char_p),
        ("output_path", c_char_p),
        ("sass_options", SassOptions),
        ("error_status", c_int),
        ("error_message", c_char_p),
    ]

class LibSass(object):
    def __init__(self):
        self.clib = None

    def _load(self):
        if self.clib is None:
            self.clib = cdll.LoadLibrary('sass.so')
            self.clib.sass_new_context.restype = POINTER(SassContext)
            self.clib.sass_new_file_context.restype = POINTER(SassFileContext)
            self.clib.sass_new_folder_context.restype = POINTER(SassFolderContext)

            self.clib.sass_compile.restype = c_int
            self.clib.sass_compile.argtypes = [POINTER(SassContext)]

            self.clib.sass_compile_file.restype = c_int
            self.clib.sass_compile_file.argtypes = [POINTER(SassFileContext)]

            self.clib.sass_compile_folder.restype = c_int
            self.clib.sass_compile_folder.argtypes = [POINTER(SassFolderContext)]

    def __getattribute__(self, name):
        attr = object.__getattribute__(self, name)
        if hasattr(attr, '__call__') and name != "_load":
            def load_wrapper(*args, **kwargs):
                self._load()
                return attr(*args, **kwargs)
            return load_wrapper
        else:
            return attr

    def sass_new_context(self):
        return self.clib.sass_new_context()

    def sass_new_file_context(self):
        return self.clib.sass_new_file_context()

    def sass_new_folder_context(self):
        return self.clib.sass_new_folder_context()

    def compile(self, ctx):
        return self.clib.sass_compile(ctx)

    def compile_file(self, ctx):
        return self.clib.sass_compile_file(ctx)

    def compile_folder(self, ctx):
        return self.clib.sass_compile_folder(ctx)

SASS_CLIB = LibSass()

def compile_str(contents):
    """Compiles a SASS string

    :param str contents: The SASS contents to compile
    :returns: The compiled CSS

    """
    ctx = SASS_CLIB.sass_new_context()
    ctx.contents.source_string = c_char_p(contents)

    SASS_CLIB.compile(ctx)

    return ctx.contents.output_string



from .events import FileSystemEvents

from watchdog.observers import Observer

def watch(source_path, dest_path):
    # Setup Watchdog
    handler = FileSystemEvents(source_path, dest_path)

    # Always run initial compile
    handler.compile_scss()

    observer = Observer(timeout=5000)
    observer.schedule(handler, path=source_path, recursive=True)
    observer.start()




import os
import sys

from setuptools import Extension, setup
from setuptools.command.test import test as TestCommand

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

execfile('pylibsass/version.py')

class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest, sys

        errno = pytest.main(self.test_args)
        sys.exit(errno)

libsass_sources = [
    'libsass/constants.cpp', 'libsass/context.cpp', 'libsass/functions.cpp', 'libsass/document.cpp',
    'libsass/document_parser.cpp', 'libsass/eval_apply.cpp', 'libsass/node.cpp',
    'libsass/node_factory.cpp', 'libsass/node_emitters.cpp', 'libsass/prelexer.cpp',
    'libsass/selector.cpp', 'libsass/sass_interface.cpp', 'libsass/sass_values.cpp',
]

libsass_headers = [
    'libsass/backtrace.hpp', 'libsass/context.hpp', 'libsass/environment.hpp',
    'libsass/eval_apply.hpp', 'libsass/node.hpp', 'libsass/prelexer.hpp',
    'libsass/sass_values.h', 'libsass/color_names.hpp', 'libsass/constants.hpp',
    'libsass/document.hpp', 'libsass/error.hpp', 'libsass/functions.hpp',
    'libsass/node_factory.hpp', 'libsass/sass_interface.h', 'libsass/selector.hpp',
]

sass_extension = Extension(
    'sass',
    libsass_sources,
    define_macros=[('LIBSASS_PYTHON_VERSION', __version__)],
    depends=libsass_headers,
    extra_compile_args=['-Wall', '-O2', '-fPIC'],
    extra_link_args=['-fPIC'],
)

with open("README.rst", 'r') as readme_file:
    readme = readme_file.read()

with open("LICENSE", 'r') as license_file:
    license = license_file.read()

setup(
    name='pylibsass',
    version=__version__,
    url='http://github.com/rsenk330/pylibsass',
    license=license,
    author='Ryan Senkbeil',
    author_email='rsenk330@gmail.com',
    description='Python wrapper for libsass',
    long_description=readme,
    ext_modules=[sass_extension],
    packages=['pylibsass'],
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    cmdclass = {'test': PyTest},
    tests_require=[
        'pytest',
        'mock',
    ],
    install_requires=[
        'watchdog',
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)
